package com.mcwl.web.controller.resource;

import cn.hutool.core.bean.BeanUtil;
import com.mcwl.common.annotation.RepeatSubmit;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.dto.ModelImageRes;
import com.mcwl.resource.domain.vo.ModelImageVo;
import com.mcwl.resource.service.ModelImageLikeService;
import com.mcwl.resource.service.ModelImageService;
import com.mcwl.system.service.ISysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 图片
 */
@Api(tags = "图片")
@RestController
@RequestMapping("/image")
@RequiredArgsConstructor
public class ModelImageController {

    private final ModelImageService modelImageService;

    private final ModelImageLikeService modelImageLikeService;

    private final ISysUserService sysUserService;


    /**
     * 图片列表
     */
    @ApiOperation(value = "图片列表")
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody PageDomain pageDomain) {
        ModelImagePageRes imagePageRes = new ModelImagePageRes();
        BeanUtil.copyProperties(pageDomain, imagePageRes);
        return modelImageService.listByPage(imagePageRes);
    }

    /**
     * 图片详情
     */
    @ApiOperation(value = "图片详情")
    @GetMapping("/detail/{imageId}")
    public AjaxResult detail(@PathVariable @NotNull(message = "图片id不能为空") Long imageId) {
        ModelImageVo modelImageVo = modelImageService.getDetail(imageId);
        return AjaxResult.success(modelImageVo);
    }

    /**
     * 图片删除
     */
    @ApiOperation(value = "图片删除")
    @GetMapping("/delete/{imageId}")
    public AjaxResult delete(@PathVariable @NotNull(message = "图片id不能为空") Long imageId) {
        modelImageService.removeById(imageId);
        return AjaxResult.success();
    }

    /**
     * 图片修改
     */
    @ApiOperation(value = "图片修改")
    @PostMapping("/update")
    public AjaxResult update(@RequestBody ModelImageRes modelImageRes) {
        modelImageService.updateById(modelImageRes);
        return AjaxResult.success();
    }


    /**
     * 图片发布
     */
    @ApiOperation(value = "图片发布")
    @PostMapping("/publish")
    public AjaxResult publish(@RequestBody ModelImageRes modelImageRes) {
        modelImageService.publish(modelImageRes);
        return AjaxResult.success();
    }

    /**
     * 图片点赞/取消
     */
    @ApiOperation(value = "图片点赞/取消")
    @RepeatSubmit
    @GetMapping("/imageLike/{imageId}")
    public AjaxResult like(@PathVariable @NotNull(message = "图片id不能为空") Long imageId) {
        modelImageLikeService.like(imageId);
        return AjaxResult.success();
    }


    /**
     * 设置图片置顶
     *
     * @param id
     * @param isTop
     * @return
     */
    @ApiOperation(value = "设置图片置顶状态")
    @GetMapping("/{id}/top")
    public AjaxResult setModelImageTop(@PathVariable Long id, @RequestParam boolean isTop) {
        modelImageLikeService.setModelImageTop(id, isTop);
        return AjaxResult.success();
    }


    /**
     * 获取置顶的图片列表
     *
     * @return
     */
    @ApiOperation(value = "获取置顶的图片列表")
    @GetMapping
    public ResponseEntity<List<ModelImage>> fetchModelImagesSortedByTopStatus() {
        List<ModelImage> models = modelImageLikeService.fetchModelImageSortedByTopStatus();
        return ResponseEntity.ok(models);
    }

}
