package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.dto.ModelImageCommentRes;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.mapper.ModelImageCommentMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.service.ModelImageCommentService;
import com.mcwl.resource.service.ModelImageService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 图片评论实现
 */
@Service
@RequiredArgsConstructor
public class ModelImageCommentServiceImpl extends ServiceImpl<ModelImageCommentMapper, ModelImageComment> implements ModelImageCommentService {

    private final ISysUserService sysUserService;

    @Override
    public void comment(ModelImageCommentRes modelImageCommentRes) {
        Long parentId = modelImageCommentRes.getParentId();

        if (Objects.nonNull(parentId)) {
            ModelImageComment mic = baseMapper.selectById(parentId);
            if (Objects.isNull(mic)) {
                return;
            }
        }

        ModelImageComment modelImageComment = new ModelImageComment();
        BeanUtil.copyProperties(modelImageCommentRes, modelImageComment);
        modelImageComment.setUserId(SecurityUtils.getUserId());
        baseMapper.insert(modelImageComment);

    }

    /**
     * 获取评论
     *
     * @param imageId 图片id
     * @return 评论区
     */
    @Override
    public List<ModelImageCommentVo> getComment(Long imageId) {

        List<ModelImageCommentVo> modelImageCommentVoList = new ArrayList<>();

        // 查询所有父评论
        LambdaQueryWrapper<ModelImageComment> lqw = new LambdaQueryWrapper<>();
        lqw.eq(ModelImageComment::getModelImageId, imageId)
                .isNull(ModelImageComment::getParentId)
                .orderByDesc(ModelImageComment::getCreateTime);
        // 添加父评论
        List<ModelImageComment> modelImageCommentList = baseMapper.selectList(lqw);
        for (ModelImageComment modelImageComment : modelImageCommentList) {
            ModelImageCommentVo modelImageCommentVo = getModelImageCommentVo(modelImageComment);
            modelImageCommentVoList.add(modelImageCommentVo);
        }

        return modelImageCommentVoList;
    }


    /**
     * 构建ModelImageCommentVo对象
     *
     * @param modelImageComment 父评论对象
     * @return ModelImageCommentVo对象
     */
    @NotNull
    private ModelImageCommentVo getModelImageCommentVo(ModelImageComment modelImageComment) {
        Long userId = modelImageComment.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);

        // 构建ModelImageCommentVo对象
        ModelImageCommentVo modelImageCommentVo = new ModelImageCommentVo();
        modelImageCommentVo.setUserId(userId);
        modelImageCommentVo.setUserName(sysUser.getUserName());
        modelImageCommentVo.setUserAvatar(sysUser.getAvatar());
        modelImageCommentVo.setCommentId(modelImageComment.getId());
        modelImageCommentVo.setContent(modelImageComment.getContent());
        // 查询子评论
        modelImageCommentVo.setContentList(getContentList(modelImageComment.getId()));
        modelImageCommentVo.setLikeNum(modelImageComment.getLikeNum());
        modelImageCommentVo.setCreateTime(modelImageComment.getCreateTime());
        return modelImageCommentVo;
    }


    /**
     * 递归查询子评论
     *
     * @param modelImageCommentId 父评论id
     * @return List<ModelImageCommentVo>
     */
    private List<ModelImageCommentVo> getContentList(Long modelImageCommentId) {
        List<ModelImageCommentVo> modelImageCommentVoList = new ArrayList<>();
        if (Objects.isNull(modelImageCommentId)) {
            return modelImageCommentVoList;
        }

        // 查询子评论
        LambdaQueryWrapper<ModelImageComment> lqw = new LambdaQueryWrapper<ModelImageComment>()
                .eq(ModelImageComment::getParentId, modelImageCommentId)
                .orderByDesc(ModelImageComment::getCreateTime);

        List<ModelImageComment> modelImageCommentList = baseMapper.selectList(lqw);

        for (ModelImageComment modelImageComment : modelImageCommentList) {
            Long userId = modelImageComment.getUserId();
            SysUser sysUser = sysUserService.selectUserById(userId);
            ModelImageCommentVo modelImageCommentVo = new ModelImageCommentVo();
            modelImageCommentVo.setUserId(userId);
            modelImageCommentVo.setUserName(sysUser.getUserName());
            modelImageCommentVo.setUserAvatar(sysUser.getAvatar());
            modelImageCommentVo.setCommentId(modelImageComment.getId());
            modelImageCommentVo.setContent(modelImageComment.getContent());
            modelImageCommentVo.setLikeNum(modelImageComment.getLikeNum());
            modelImageCommentVo.setCreateTime(modelImageComment.getCreateTime());


            modelImageCommentVoList.add(modelImageCommentVo);
        }
        return modelImageCommentVoList;
    }



}
