package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.DictConstants;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.common.utils.baidu.BaiduCensor;
import com.mcwl.resource.domain.WorkFlow;
import com.mcwl.resource.domain.WorkFlowVersion;
import com.mcwl.resource.domain.dto.AddRequestWorkFlow;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.request.RequestWorkFlow;
import com.mcwl.resource.domain.vo.PageVo;
import com.mcwl.resource.domain.vo.WorkFlowVo;
import com.mcwl.resource.mapper.WorkFlowMapper;
import com.mcwl.resource.mapper.WorkFlowVersionMapper;
import com.mcwl.resource.service.ToActivityService;
import com.mcwl.resource.service.WorkFlowService;
import com.mcwl.system.init.DictInit;
import com.mcwl.system.service.ISysUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 工作流 业务实现层
 *
 * @author DaiZibo
 * @date 2025/1/9
 * @apiNote
 */

@Slf4j
@Service
public class WorkFlowServiceImpl extends ServiceImpl<WorkFlowMapper, WorkFlow> implements WorkFlowService {

    @Autowired
    private WorkFlowMapper flowMapper;

    @Autowired
    private WorkFlowVersionMapper workFlowVersionMapper;

    @Autowired
    private ToActivityService toActivityService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;


    @Override
    public AjaxResult addWorkFlow(AddRequestWorkFlow addRequestWorkFlo) {

        RequestWorkFlow requestWorkFlow = new RequestWorkFlow();
        BeanUtil.copyProperties(addRequestWorkFlo, requestWorkFlow);

        //获取封面图
        String filePath = requestWorkFlow.getWorkFlowVersionList().get(0).getFilePath();

        String[] split = filePath.split(",");

        WorkFlow workFlow = requestWorkFlow.getWorkFlow();
        workFlow.setUserId(SecurityUtils.getUserId());
        workFlow.setCoverPath(split[0]);
        workFlow.setCreateTime(new Date());
        //添加模型表数据
        flowMapper.insert(workFlow);

        log.info("获取到的入参:{}", requestWorkFlow.getWorkFlow());

        //批量添加版本
        workFlowVersionMapper.addWorkFlowVersion(requestWorkFlow.getWorkFlow(), requestWorkFlow.getWorkFlowVersionList());

        audit(requestWorkFlow);



        return AjaxResult.success("添加成功，等待审核");
    }

    private void audit(RequestWorkFlow requestWorkFlow) {
        // 执行审核操作
        threadPoolTaskExecutor.submit(() -> {
            WorkFlow workFlow = requestWorkFlow.getWorkFlow();

            if (workFlow != null){

                if (workFlow.getWorkflowName() != null){

                    //审核名称
                    String s = BaiduCensor.TextCensor(workFlow.getWorkflowName());
                    // 解析 JSON 字符串
                    JSONObject jsonObject = JSONObject.parseObject(s);
                    // 获取 'conclusion' 字段的值
                    String conclusion = jsonObject.getString("conclusion");

                    if (conclusion.equals("不合规")) {

                        //更改状态,添加原因 -> 结束任务
                        // 获取 'data' 数组
                        JSONArray dataArray = jsonObject.getJSONArray("data");

                        // 存储所有的失败原因
                        List<String> failureReasons = new ArrayList<>();
                        // 遍历 'data' 数组中的每个元素
                        for (Object itemObj : dataArray) {
                            JSONObject item = (JSONObject) itemObj;
                            String msg = item.getString("msg");
                            failureReasons.add(msg);
                        }
                        // 使用逗号拼接所有的失败原因
                        String concatenatedReasons = String.join(", ", failureReasons);
                        workFlow.setAuditText(concatenatedReasons);

                        //修改状态以及失败原因
                        workFlow.setAuditStatus(4);
                        flowMapper.updateById(workFlow);
                        log.info("工作流审核未通过");
                        //结束任务
                        return;
                    }

                    //修改为合格
                    workFlow.setAuditText("");
                    workFlow.setAuditStatus(workFlow.getJurisdiction());
                    flowMapper.updateById(workFlow);
                }
            }


            List<WorkFlowVersion> workFlowVersionList = requestWorkFlow.getWorkFlowVersionList();

            if (workFlowVersionList != null){
                //审核版本
                for (WorkFlowVersion workFlowVersion : workFlowVersionList) {
                    workFlowVersion.setWorkFlowId(workFlow.getId());
                    //版本名称
                    String s1 = BaiduCensor.TextCensor(workFlowVersion.getVersionName() + "," + workFlowVersion.getVersionDescription());
                    // 解析 JSON 字符串
                    JSONObject jsonObject1 = JSONObject.parseObject(s1);
                    // 获取 'conclusion' 字段的值
                    String conclusion1 = jsonObject1.getString("conclusion");
                    if (conclusion1.equals("不合规")) {

                        //更改版本状态->跳出循环 审核下一个版本
                        // 获取 'data' 数组
                        JSONArray dataArray1 = jsonObject1.getJSONArray("data");
                        // 存储所有的失败原因
                        List<String> failureReasons = new ArrayList<>();
                        // 遍历 'data' 数组中的每个元素
                        for (Object itemObj : dataArray1) {
                            JSONObject item = (JSONObject) itemObj;
                            String msg = item.getString("msg");
                            failureReasons.add(msg);
                        }
                        // 使用逗号拼接所有的失败原因
                        String concatenatedReasons = String.join(", ", failureReasons);
                        workFlowVersion.setAuditText(concatenatedReasons);
                        workFlowVersion.setAuditStatus(4);
                        //校验id是否存在 区别修改审核或者新增审核
                        if (workFlowVersion.getId() != null){
                            workFlowVersionMapper.updateById(workFlowVersion);
                        }else {
                            workFlowVersionMapper.updateByName(workFlowVersion);
                        }
                        log.info("版本审核未通过");
                        continue;
                    }

                    //审核版本图片
                    String[] split = workFlowVersion.getFilePath().split(",");
                    for (String path : split) {
                        String s2 = BaiduCensor.ImageCnesor(path);
                        JSONObject jsonObject2 = JSONObject.parseObject(s2);
                        if (jsonObject2.getString("conclusion").equals("不合规")) {

                            //修改状态->跳出循环 判断下一个版本
                            // 获取 'data' 数组
                            JSONArray dataArray2 = jsonObject2.getJSONArray("data");
                            // 存储所有的失败原因
                            List<String> failureReasons = new ArrayList<>();
                            // 遍历 'data' 数组中的每个元素
                            for (Object itemObj : dataArray2) {
                                JSONObject item = (JSONObject) itemObj;
                                String msg = item.getString("msg");
                                failureReasons.add(msg);
                            }
                            // 使用逗号拼接所有的失败原因
                            String concatenatedReasons = String.join(", ", failureReasons);
                            workFlowVersion.setAuditText(concatenatedReasons);
                            workFlowVersion.setAuditStatus(4);
                            //校验id是否存在 区别修改审核或者新增审核
                            if (workFlowVersion.getId() != null){
                                workFlowVersionMapper.updateById(workFlowVersion);
                            }else {
                                workFlowVersionMapper.updateByName(workFlowVersion);
                            }
                            log.info("图片审核未通过");
                            break;
                        }
                    }

                    //修改版本成功审核状态
                    workFlow.setAuditText("");
                    workFlowVersion.setAuditStatus(1);
                    workFlowVersionMapper.updateByName(workFlowVersion);
                    log.info("全部通过审核");
                }
            }

        });
    }

    @Override
    public void updateWorkFlow(RequestWorkFlow requestWorkFlow) {
        WorkFlow workFlows = requestWorkFlow.getWorkFlow();
        //修改工作流的信息
        if (ObjectUtils.isEmpty(workFlows)) {
            return;
        }
        if (StringUtils.isNotNull(workFlows.getId())) {

            workFlows.setUpdateTime(new Date());
            workFlows.setAuditStatus(3);
//            flowMapper.updateWorkFlow(requestWorkFlow.getWorkFlow());
            flowMapper.updateById(requestWorkFlow.getWorkFlow());
        }

        //修改工作流版本的信息
        if (requestWorkFlow.getWorkFlowVersionList().size() != 0) {

            //批量修改
            for (WorkFlowVersion workFlowVersion : requestWorkFlow.getWorkFlowVersionList()) {
                workFlowVersion.setAuditStatus(3);
//                workFlowVersionMapper.updateWorkFlowVersion(workFlowVersion);
                workFlowVersionMapper.updateById(workFlowVersion);
            }

            WorkFlow workFlow = WorkFlow.builder().id(requestWorkFlow.getWorkFlowVersionList().get(0).getWorkFlowId())
                    .updateTime(new Date()).build();
            //更新时间
            flowMapper.updateById(workFlow);
        }

        //审核修改后的数据
        audit(requestWorkFlow);

    }

    @Override
    public void deleteWorkFlow(Long id) {

        WorkFlow workFlow = WorkFlow.builder().id(id)
                .delFlag(2).build();

        flowMapper.updateById(workFlow);
    }

    @Override
    public AjaxResult selectWorkFlow(PageVo pageVo) {

        Page<WorkFlow> page = new Page<>(pageVo.getPageNumber(), pageVo.getPageSize());

        LambdaQueryWrapper<WorkFlow> lambdaQueryWrapper = Wrappers.<WorkFlow>lambdaQuery()
                .like(StringUtils.isNotBlank(pageVo.getName()), WorkFlow::getWorkflowName, pageVo.getName())
                .eq(WorkFlow::getDelFlag, 0);

        if (pageVo.getOrder() == 1) {
            lambdaQueryWrapper.orderByDesc(WorkFlow::getUseNumber);
        } else {
            lambdaQueryWrapper.orderByDesc(WorkFlow::getId);
        }

        lambdaQueryWrapper.select(WorkFlow::getId, WorkFlow::getWorkflowName, WorkFlow::getCoverPath);

        return AjaxResult.success(flowMapper.selectPage(page, lambdaQueryWrapper));
    }

    @Override
    public AjaxResult selectWorkFlowById(Long id,Integer type) {

        //查询详情
        WorkFlow workFlow = flowMapper.selectById(id);
        if (workFlow == null){
            return AjaxResult.error("工作流不存在");
        }

        if (type == 1){

            //类别
            if (StringUtils.isNotEmpty(workFlow.getType())) {
                ArrayList<String> strings = new ArrayList<>();
                String[] split = workFlow.getType().split(",");
                for (String s : split) {
                    if (s != "") {
                        strings.add(DictInit.getDictValue(DictConstants.WORK_FLOW_TYPE_CHILD, s));
                    }
                }
                workFlow.setTypeList(strings);
            }

        }

        return AjaxResult.success(workFlow);
    }


    @Override
    public TableDataInfo listByPage(ModelImagePageRes imagePageRes) {

        Page<WorkFlow> page = new Page<>(imagePageRes.getPageNum(), imagePageRes.getPageSize());
        if (StringUtils.isEmpty(imagePageRes.getOrderByColumn())) {
            imagePageRes.setOrderByColumn("create_time");
        }
        // 设置排序
        // 设置排序
        List<OrderItem> orderItemList = new ArrayList<>();
        orderItemList.add(OrderItem.desc("is_top"));
        orderItemList.add(OrderItem.desc(imagePageRes.getOrderByColumn()));
        page.addOrder(orderItemList);

        LambdaQueryWrapper<WorkFlow> lqw = new LambdaQueryWrapper<>();
        lqw.eq(imagePageRes.getStatus() != null && imagePageRes.getStatus() != 0 , WorkFlow::getAuditStatus, imagePageRes.getStatus())
                .eq(WorkFlow::getUserId, SecurityUtils.getUserId())
                .ge(imagePageRes.getStartTime() != null, WorkFlow::getCreateTime, imagePageRes.getStartTime())
                .le(imagePageRes.getEndTime() != null, WorkFlow::getCreateTime, imagePageRes.getEndTime())
                .eq(WorkFlow::getDelFlag,0);

        baseMapper.selectPage(page, lqw);
        // 获取分页数据
        List<WorkFlow> workFlowList = page.getRecords();

        // WorkFlow数据转为WorkFlowVo
        List<WorkFlowVo> workFlowVoList = new ArrayList<>();
        for (WorkFlow workFlow : workFlowList) {
            WorkFlowVo workFlowVo = new WorkFlowVo();
            BeanUtil.copyProperties(workFlow, workFlowVo);
            // 获取用户信息
            SysUser sysUser = sysUserService.selectUserById(workFlow.getUserId());
            workFlowVo.setUserId(sysUser.getUserId());
            workFlowVo.setUserName(sysUser.getUserName());
            workFlowVo.setUserAvatar(sysUser.getAvatar());
            workFlowVoList.add(workFlowVo);
        }

        // 封装分页数据
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(workFlowVoList);
        rspData.setTotal(page.getTotal());

        return rspData;

    }

    @Override
    public void setworkFlowTop(Long id, boolean isTop) {
        flowMapper.setworkFlowTop(id,isTop?1:0);
    }

    @Override
    public List<WorkFlow> fetchWorkFlowSortedByTopStatus() {
        return flowMapper.fetchWorkFlowSortedByTopStatus();
    }


}
